/*
 * File:   gs_path.c
 * Author: eis_os (cim bytetransfer.de)
 * 
 * Shared code for reading paths file
 * 
 * last update: 01.10.2011
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gs.h"
#include "gs_math.h"
#include "gs_path.h"
#include "gs_filefunc.h"


struct path_data_ll* read_paths_file(const char *filename)
{
    struct parser *parser;
    struct path_data_ll *p = NULL; // current path
    struct path_data_ll *path_list_start = NULL; // start of list

    if (!file_exists(filename)) {
          printf("reading: '%s' skipped, file not found...\n", filename);
          return path_list_start;
    }
    
    parser = init_parser(filename);
    while(parser->line) {
	char *line = parser->line;
        if(*line)
        if(*line == '#') {
            // ignoring comments
        } else if(strncmp(line, "name ", 5) == 0) {
	    struct path_data_ll *newp = calloc(1, sizeof(struct path_data_ll));
            strncpy(newp->p.name, line+5, 32);
	    if(p)
		p->next = newp;
	    else
		path_list_start = newp;
	    p = newp;
	} else if(strncmp(line, "pos ", 4) == 0) {
	    if(!p)
                die("At line %u in '%s' missing name statement\n", parser->nline, filename);
            char *atpos = line+4;
            p->p.pos.x = read_float(atpos, &atpos, parser);
            p->p.pos.y = read_float(atpos, &atpos, parser);
            p->p.pos.z = read_float(atpos, &atpos, parser);
        } else if(strncmp(line, "quat ", 5) == 0) {
            if(!p)
                die("At line %u in '%s' missing name statement\n", parser->nline, filename);
            if (p->p.quat.x ||  p->p.quat.y ||  p->p.quat.z || p->p.quat.w) {
                printf("Notice at line %u in '%s' quat already defined\n", parser->nline, filename);
            }
            char *atpos = line+5;
            p->p.quat.x = read_float(atpos, &atpos, parser);
            p->p.quat.y = read_float(atpos, &atpos, parser);
            p->p.quat.z = read_float(atpos, &atpos, parser);
            p->p.quat.w = read_float(atpos, &atpos, parser);
	} else if(strncmp(line, "rotation ", 9) == 0) {
	    if(!p)
                die("At line %u in '%s' missing name statement\n", parser->nline, filename);
            if (p->p.quat.x ||  p->p.quat.y ||  p->p.quat.z || p->p.quat.w) {
                printf("Notice at line %u in '%s' quat already defined\n", parser->nline, filename);
            }
            char *atpos = line+9;
            gs_vec radrot;
            gs_quat quat;
            radrot.x = read_float(atpos, &atpos, parser) * DEGTORAD;
            radrot.y = read_float(atpos, &atpos, parser) * DEGTORAD;
            radrot.z = read_float(atpos, &atpos, parser) * DEGTORAD;
            
            p->p.quat = euler_to_quat(radrot);
        } else if (*line = '#') { 
            
	} else {
                die("Unknown statement at line %u in '%s'\n", parser->nline, filename);
        }
	parser_read_line(parser); // read next line
    }
    close(parser->fd);
    free(parser);
    return path_list_start;
}

void write_obj_path_file_combined(const char *obj_path_file, int force_overwrite, struct path_data_ll *pd, const char *toolname) {
    unsigned int i,j;
    FILE *f;

    if (pd == NULL) return;
    
    printf("writing obj path file: %s\n", obj_path_file);
    if(!force_overwrite && file_exists(obj_path_file))
        die("'%s' already exists. Remove or use -f to force overwrite\n",
            obj_path_file);

    f = fopen(obj_path_file, "w");
    if(!f)
	die("fopen(%s): %s\n", obj_path_file, strerror(errno));
    
    fprintf(f, "# path obj created by %s \n", toolname);
    //fprintf(f, "o %s\n", obj->info.name);
    fprintf(f, "mtllib (None)\n");
    fprintf(f, "o gsPaths\n");
   
    while(pd > 0) {

        fprintf(f, "g %s\n", pd->p.name);
               
        gs_vec position = pd->p.pos;
        gs_quat rotation = pd->p.quat;
      
        gs_vec marker[4] = {
            {0.5, 0.5, 0},
            {0.5, -0.5, 0},
            {-0.5, 0, 0},
            {0, 0, 0},
        };
        
        for (i = 0; i < 4; i++) {
                gs_vec point;
              //  point = vec_add(position, marker[i]);
                fprintf(f, "# pos %g %g %g\n", position.x, position.y, position.z);
                point = quat_multiply_vec(rotation, marker[i]);
                point = vec_add(position, point);
                //point = quat_multiply_vec(rotation, vec_add(position, marker[i]));
                fprintf(f, "v %g %g %g\n", point.x, point.y, point.z);
        }
        fprintf(f, "f -1 -2 -3\n");
        fprintf(f, "f -1 -4 -2\n");
        
        pd = pd->next;
    }
    fclose(f);
}